@@ -1,9 +1,13 @@
-from typing import Callable, Dict, List, Optional, Union, TypeVar, cast
 from functools import partial
+from typing import Callable, Dict, List, Optional, TypeVar, Union, cast
 
+from ..error import GraphQLError, GraphQLSyntaxError
 from .ast import (
     ArgumentNode,
     BooleanValueNode,
+    ConstArgumentNode,
+    ConstDirectiveNode,
+    ConstValueNode,
     DefinitionNode,
     DirectiveDefinitionNode,
     DirectiveNode,
@@ -21,14 +25,14 @@ from .ast import (
     InputObjectTypeDefinitionNode,
     InputObjectTypeExtensionNode,
     InputValueDefinitionNode,
-    IntValueNode,
     InterfaceTypeDefinitionNode,
     InterfaceTypeExtensionNode,
+    IntValueNode,
     ListTypeNode,
     ListValueNode,
     Location,
-    NameNode,
     NamedTypeNode,
+    NameNode,
     NonNullTypeNode,
     NullValueNode,
     ObjectFieldNode,
@@ -45,8 +49,8 @@ from .ast import (
     SelectionNode,
     SelectionSetNode,
     StringValueNode,
+    Token,
     TypeNode,
-    TypeSystemDefinitionNode,
     TypeSystemExtensionNode,
     UnionTypeDefinitionNode,
     UnionTypeExtensionNode,
@@ -55,14 +59,11 @@ from .ast import (
     VariableNode,
 )
 from .directive_locations import DirectiveLocation
-from .ast import Token
 from .lexer import Lexer, is_punctuator_token_kind
-from .source import Source
+from .source import Source, is_source
 from .token_kind import TokenKind
-from ..error import GraphQLError, GraphQLSyntaxError
-from ..pyutils import inspect
 
-__all__ = ["parse", "parse_type", "parse_value"]
+__all__ = ["parse", "parse_type", "parse_value", "parse_const_value"]
 
 T = TypeVar("T")
 
@@ -70,19 +71,28 @@ SourceType = Union[Source, str]
 
 
 def parse(
-    source: SourceType, no_location=False, experimental_fragment_variables=False
+    source: SourceType,
+    no_location: bool = False,
+    max_tokens: Optional[int] = None,
+    allow_legacy_fragment_variables: bool = False,
 ) -> DocumentNode:
     """Given a GraphQL source, parse it into a Document.
 
     Throws GraphQLError if a syntax error is encountered.
 
     By default, the parser creates AST nodes that know the location in the source that
-    they correspond to. The ``no_location`` option disables that behavior for
-    performance or testing.
+    they correspond to. Setting the ``no_location`` parameter to False disables that
+    behavior for performance or testing.
 
-    Experimental features:
+    Parser CPU and memory usage is linear to the number of tokens in a document,
+    however in extreme cases it becomes quadratic due to memory exhaustion.
+    Parsing happens before validation, so even invalid queries can burn lots of
+    CPU time and memory. To prevent this, you can set a maximum number of tokens
+    allowed within a document using the ``max_tokens`` parameter.
 
-    If ``experimental_fragment_variables`` is set to ``True``, the parser will
+    Legacy feature (will be removed in v3.3):
+
+    If ``allow_legacy_fragment_variables`` is set to ``True``, the parser will
     understand and parse variable definitions contained in a fragment definition.
     They'll be represented in the
     :attr:`~graphql.language.FragmentDefinitionNode.variable_definitions` field
@@ -97,13 +107,17 @@ def parse(
     parser = Parser(
         source,
         no_location=no_location,
-        experimental_fragment_variables=experimental_fragment_variables,
+        max_tokens=max_tokens,
+        allow_legacy_fragment_variables=allow_legacy_fragment_variables,
     )
     return parser.parse_document()
 
 
 def parse_value(
-    source: SourceType, no_location=False, experimental_fragment_variables=False
+    source: SourceType,
+    no_location: bool = False,
+    max_tokens: Optional[int] = None,
+    allow_legacy_fragment_variables: bool = False,
 ) -> ValueNode:
     """Parse the AST for a given string containing a GraphQL value.
 
@@ -113,12 +127,13 @@ def parse_value(
     isolation of complete GraphQL documents.
 
     Consider providing the results to the utility function:
-    :func:`~graphql.value_from_ast`.
+    :func:`~graphql.utilities.value_from_ast`.
     """
     parser = Parser(
         source,
         no_location=no_location,
-        experimental_fragment_variables=experimental_fragment_variables,
+        max_tokens=max_tokens,
+        allow_legacy_fragment_variables=allow_legacy_fragment_variables,
     )
     parser.expect_token(TokenKind.SOF)
     value = parser.parse_value_literal(False)
@@ -126,8 +141,34 @@ def parse_value(
     return value
 
 
+def parse_const_value(
+    source: SourceType,
+    no_location: bool = False,
+    max_tokens: Optional[int] = None,
+    allow_legacy_fragment_variables: bool = False,
+) -> ConstValueNode:
+    """Parse the AST for a given string containing a GraphQL constant value.
+
+    Similar to parse_value, but raises a arse error if it encounters a variable.
+    The return type will be a constant value.
+    """
+    parser = Parser(
+        source,
+        no_location=no_location,
+        max_tokens=max_tokens,
+        allow_legacy_fragment_variables=allow_legacy_fragment_variables,
+    )
+    parser.expect_token(TokenKind.SOF)
+    value = parser.parse_const_value_literal()
+    parser.expect_token(TokenKind.EOF)
+    return value
+
+
 def parse_type(
-    source: SourceType, no_location=False, experimental_fragment_variables=False
+    source: SourceType,
+    no_location: bool = False,
+    max_tokens: Optional[int] = None,
+    allow_legacy_fragment_variables: bool = False,
 ) -> TypeNode:
     """Parse the AST for a given string containing a GraphQL Type.
 
@@ -137,12 +178,13 @@ def parse_type(
     in isolation of complete GraphQL documents.
 
     Consider providing the results to the utility function:
-    :func:`~graphql.value_from_ast`.
+    :func:`~graphql.utilities.value_from_ast`.
     """
     parser = Parser(
         source,
         no_location=no_location,
-        experimental_fragment_variables=experimental_fragment_variables,
+        max_tokens=max_tokens,
+        allow_legacy_fragment_variables=allow_legacy_fragment_variables,
     )
     parser.expect_token(TokenKind.SOF)
     type_ = parser.parse_type_reference()
@@ -151,24 +193,39 @@ def parse_type(
 
 
 class Parser:
+    """GraphQL AST parser.
+
+    This class is exported only to assist people in implementing their own parsers
+    without duplicating too much code and should be used only as last resort for cases
+    such as experimental syntax or if certain features couldn't be contributed upstream.
+
+    It's still part of the internal API and is versioned, so any changes to it are never
+    considered breaking changes. If you still need to support multiple versions of the
+    library, please use the `__version_info__` variable for version detection.
+    """
 
     _lexer: Lexer
-    _no_Location: bool
-    _experimental_fragment_variables: bool
+    _no_location: bool
+    _max_tokens: Optional[int]
+    _allow_legacy_fragment_variables: bool
+    _token_counter: int
 
     def __init__(
         self,
         source: SourceType,
         no_location: bool = False,
-        experimental_fragment_variables: bool = False,
+        max_tokens: Optional[int] = None,
+        allow_legacy_fragment_variables: bool = False,
     ):
-        if isinstance(source, str):
-            source = Source(source)
-        elif not isinstance(source, Source):
-            raise TypeError(f"Must provide Source. Received: {inspect(source)}.")
+        source = (
+            cast(Source, source) if is_source(source) else Source(cast(str, source))
+        )
+
         self._lexer = Lexer(source)
         self._no_location = no_location
-        self._experimental_fragment_variables = experimental_fragment_variables
+        self._max_tokens = max_tokens
+        self._allow_legacy_fragment_variables = allow_legacy_fragment_variables
+        self._token_counter = 0
 
     def parse_name(self) -> NameNode:
         """Convert a name lex token into a name parse node."""
@@ -185,22 +242,20 @@ class Parser:
             loc=self.loc(start),
         )
 
-    _parse_definition_method_names: Dict[str, str] = {
+    _parse_type_system_definition_method_names: Dict[str, str] = {
+        "schema": "schema_definition",
+        "scalar": "scalar_type_definition",
+        "type": "object_type_definition",
+        "interface": "interface_type_definition",
+        "union": "union_type_definition",
+        "enum": "enum_type_definition",
+        "input": "input_object_type_definition",
+        "directive": "directive_definition",
+    }
+
+    _parse_other_definition_method_names: Dict[str, str] = {
         **dict.fromkeys(("query", "mutation", "subscription"), "operation_definition"),
         "fragment": "fragment_definition",
-        **dict.fromkeys(
-            (
-                "schema",
-                "scalar",
-                "type",
-                "interface",
-                "union",
-                "enum",
-                "input",
-                "directive",
-            ),
-            "type_system_definition",
-        ),
         "extend": "type_system_extension",
     }
 
@@ -208,23 +263,43 @@ class Parser:
         """Definition: ExecutableDefinition or TypeSystemDefinition/Extension
 
         ExecutableDefinition: OperationDefinition or FragmentDefinition
+
+        TypeSystemDefinition: SchemaDefinition, TypeDefinition or DirectiveDefinition
+
+        TypeDefinition: ScalarTypeDefinition, ObjectTypeDefinition,
+            InterfaceTypeDefinition, UnionTypeDefinition,
+            EnumTypeDefinition or InputObjectTypeDefinition
         """
-        if self.peek(TokenKind.NAME):
-            method_name = self._parse_definition_method_names.get(
-                cast(str, self._lexer.token.value)
+        if self.peek(TokenKind.BRACE_L):
+            return self.parse_operation_definition()
+
+        # Many definitions begin with a description and require a lookahead.
+        has_description = self.peek_description()
+        keyword_token = (
+            self._lexer.lookahead() if has_description else self._lexer.token
+        )
+
+        if keyword_token.kind is TokenKind.NAME:
+            token_name = cast(str, keyword_token.value)
+            method_name = self._parse_type_system_definition_method_names.get(
+                token_name
             )
             if method_name:
                 return getattr(self, f"parse_{method_name}")()
-        elif self.peek(TokenKind.BRACE_L):
-            return self.parse_operation_definition()
-        elif self.peek_description():
-            return self.parse_type_system_definition()
-        raise self.unexpected()
 
-    _parse_executable_definition_method_names: Dict[str, str] = {
-        **dict.fromkeys(("query", "mutation", "subscription"), "operation_definition"),
-        **dict.fromkeys(("fragment",), "fragment_definition"),
-    }
+            if has_description:
+                raise GraphQLSyntaxError(
+                    self._lexer.source,
+                    self._lexer.token.start,
+                    "Unexpected description,"
+                    " descriptions are supported only on type definitions.",
+                )
+
+            method_name = self._parse_other_definition_method_names.get(token_name)
+            if method_name:
+                return getattr(self, f"parse_{method_name}")()
+
+        raise self.unexpected(keyword_token)
 
     # Implement the parsing rules in the Operations section.
 
@@ -271,10 +346,12 @@ class Parser:
         return VariableDefinitionNode(
             variable=self.parse_variable(),
             type=self.expect_token(TokenKind.COLON) and self.parse_type_reference(),
-            default_value=self.parse_value_literal(True)
-            if self.expect_optional_token(TokenKind.EQUALS)
-            else None,
-            directives=self.parse_directives(True),
+            default_value=(
+                self.parse_const_value_literal()
+                if self.expect_optional_token(TokenKind.EQUALS)
+                else None
+            ),
+            directives=self.parse_const_directives(),
             loc=self.loc(start),
         )
 
@@ -315,35 +392,32 @@ class Parser:
             name=name,
             arguments=self.parse_arguments(False),
             directives=self.parse_directives(False),
-            selection_set=self.parse_selection_set()
-            if self.peek(TokenKind.BRACE_L)
-            else None,
+            selection_set=(
+                self.parse_selection_set() if self.peek(TokenKind.BRACE_L) else None
+            ),
             loc=self.loc(start),
         )
 
     def parse_arguments(self, is_const: bool) -> List[ArgumentNode]:
         """Arguments[Const]: (Argument[?Const]+)"""
         item = self.parse_const_argument if is_const else self.parse_argument
-        return self.optional_many(TokenKind.PAREN_L, item, TokenKind.PAREN_R)
+        return self.optional_many(
+            TokenKind.PAREN_L, cast(Callable[[], ArgumentNode], item), TokenKind.PAREN_R
+        )
 
-    def parse_argument(self) -> ArgumentNode:
-        """Argument: Name : Value"""
+    def parse_argument(self, is_const: bool = False) -> ArgumentNode:
+        """Argument[Const]: Name : Value[?Const]"""
         start = self._lexer.token
         name = self.parse_name()
 
         self.expect_token(TokenKind.COLON)
         return ArgumentNode(
-            name=name, value=self.parse_value_literal(False), loc=self.loc(start)
+            name=name, value=self.parse_value_literal(is_const), loc=self.loc(start)
         )
 
-    def parse_const_argument(self) -> ArgumentNode:
-        """Argument[Const]: Name : Value[?Const]"""
-        start = self._lexer.token
-        return ArgumentNode(
-            name=self.parse_name(),
-            value=self.expect_token(TokenKind.COLON) and self.parse_value_literal(True),
-            loc=self.loc(start),
-        )
+    def parse_const_argument(self) -> ConstArgumentNode:
+        """Argument[Const]: Name : Value[Const]"""
+        return cast(ConstArgumentNode, self.parse_argument(True))
 
     # Implement the parsing rules in the Fragments section.
 
@@ -374,9 +448,9 @@ class Parser:
         """FragmentDefinition"""
         start = self._lexer.token
         self.expect_keyword("fragment")
-        # Experimental support for defining variables within fragments changes
+        # Legacy support for defining variables within fragments changes
         # the grammar of FragmentDefinition
-        if self._experimental_fragment_variables:
+        if self._allow_legacy_fragment_variables:
             return FragmentDefinitionNode(
                 name=self.parse_fragment_name(),
                 variable_definitions=self.parse_variable_definitions(),
@@ -425,7 +499,7 @@ class Parser:
 
     def parse_string_literal(self, _is_const: bool = False) -> StringValueNode:
         token = self._lexer.token
-        self._lexer.advance()
+        self.advance_lexer()
         return StringValueNode(
             value=token.value,
             block=token.kind == TokenKind.BLOCK_STRING,
@@ -462,18 +536,18 @@ class Parser:
 
     def parse_int(self, _is_const: bool = False) -> IntValueNode:
         token = self._lexer.token
-        self._lexer.advance()
+        self.advance_lexer()
         return IntValueNode(value=token.value, loc=self.loc(token))
 
     def parse_float(self, _is_const: bool = False) -> FloatValueNode:
         token = self._lexer.token
-        self._lexer.advance()
+        self.advance_lexer()
         return FloatValueNode(value=token.value, loc=self.loc(token))
 
     def parse_named_values(self, _is_const: bool = False) -> ValueNode:
         token = self._lexer.token
         value = token.value
-        self._lexer.advance()
+        self.advance_lexer()
         if value == "true":
             return BooleanValueNode(value=True, loc=self.loc(token))
         if value == "false":
@@ -483,9 +557,21 @@ class Parser:
         return EnumValueNode(value=value, loc=self.loc(token))
 
     def parse_variable_value(self, is_const: bool) -> VariableNode:
-        if not is_const:
-            return self.parse_variable()
-        raise self.unexpected()
+        if is_const:
+            variable_token = self.expect_token(TokenKind.DOLLAR)
+            token = self._lexer.token
+            if token.kind is TokenKind.NAME:
+                var_name = token.value
+                raise GraphQLSyntaxError(
+                    self._lexer.source,
+                    variable_token.start,
+                    f"Unexpected variable '${var_name}' in constant value.",
+                )
+            raise self.unexpected(variable_token)
+        return self.parse_variable()
+
+    def parse_const_value_literal(self) -> ConstValueNode:
+        return cast(ConstValueNode, self.parse_value_literal(True))
 
     # Implement the parsing rules in the Directives section.
 
@@ -497,6 +583,9 @@ class Parser:
             append(self.parse_directive(is_const))
         return directives
 
+    def parse_const_directives(self) -> List[ConstDirectiveNode]:
+        return cast(List[ConstDirectiveNode], self.parse_directives(True))
+
     def parse_directive(self, is_const: bool) -> DirectiveNode:
         """Directive[Const]: @ Name Arguments[?Const]?"""
         start = self._lexer.token
@@ -512,10 +601,11 @@ class Parser:
     def parse_type_reference(self) -> TypeNode:
         """Type: NamedType or ListType or NonNullType"""
         start = self._lexer.token
+        type_: TypeNode
         if self.expect_optional_token(TokenKind.BRACKET_L):
-            type_ = self.parse_type_reference()
+            inner_type = self.parse_type_reference()
             self.expect_token(TokenKind.BRACKET_R)
-            type_ = ListTypeNode(type=type_, loc=self.loc(start))
+            type_ = ListTypeNode(type=inner_type, loc=self.loc(start))
         else:
             type_ = self.parse_named_type()
         if self.expect_optional_token(TokenKind.BANG):
@@ -529,30 +619,6 @@ class Parser:
 
     # Implement the parsing rules in the Type Definition section.
 
-    _parse_type_system_definition_method_names: Dict[str, str] = {
-        "schema": "schema_definition",
-        "scalar": "scalar_type_definition",
-        "type": "object_type_definition",
-        "interface": "interface_type_definition",
-        "union": "union_type_definition",
-        "enum": "enum_type_definition",
-        "input": "input_object_type_definition",
-        "directive": "directive_definition",
-    }
-
-    def parse_type_system_definition(self) -> TypeSystemDefinitionNode:
-        """TypeSystemDefinition"""
-        # Many definitions begin with a description and require a lookahead.
-        keyword_token = (
-            self._lexer.lookahead() if self.peek_description() else self._lexer.token
-        )
-        method_name = self._parse_type_system_definition_method_names.get(
-            cast(str, keyword_token.value)
-        )
-        if method_name:
-            return getattr(self, f"parse_{method_name}")()
-        raise self.unexpected(keyword_token)
-
     _parse_type_extension_method_names: Dict[str, str] = {
         "schema": "schema_extension",
         "scalar": "scalar_type_extension",
@@ -588,7 +654,7 @@ class Parser:
         start = self._lexer.token
         description = self.parse_description()
         self.expect_keyword("schema")
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         operation_types = self.many(
             TokenKind.BRACE_L, self.parse_operation_type_definition, TokenKind.BRACE_R
         )
@@ -615,7 +681,7 @@ class Parser:
         description = self.parse_description()
         self.expect_keyword("scalar")
         name = self.parse_name()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         return ScalarTypeDefinitionNode(
             description=description,
             name=name,
@@ -630,7 +696,7 @@ class Parser:
         self.expect_keyword("type")
         name = self.parse_name()
         interfaces = self.parse_implements_interfaces()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         fields = self.parse_fields_definition()
         return ObjectTypeDefinitionNode(
             description=description,
@@ -643,16 +709,11 @@ class Parser:
 
     def parse_implements_interfaces(self) -> List[NamedTypeNode]:
         """ImplementsInterfaces"""
-        types: List[NamedTypeNode] = []
-        if self.expect_optional_keyword("implements"):
-            # optional leading ampersand
-            self.expect_optional_token(TokenKind.AMP)
-            append = types.append
-            while True:
-                append(self.parse_named_type())
-                if not self.expect_optional_token(TokenKind.AMP):
-                    break
-        return types
+        return (
+            self.delimited_many(TokenKind.AMP, self.parse_named_type)
+            if self.expect_optional_keyword("implements")
+            else []
+        )
 
     def parse_fields_definition(self) -> List[FieldDefinitionNode]:
         """FieldsDefinition: {FieldDefinition+}"""
@@ -668,7 +729,7 @@ class Parser:
         args = self.parse_argument_defs()
         self.expect_token(TokenKind.COLON)
         type_ = self.parse_type_reference()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         return FieldDefinitionNode(
             description=description,
             name=name,
@@ -692,11 +753,11 @@ class Parser:
         self.expect_token(TokenKind.COLON)
         type_ = self.parse_type_reference()
         default_value = (
-            self.parse_value_literal(True)
+            self.parse_const_value_literal()
             if self.expect_optional_token(TokenKind.EQUALS)
             else None
         )
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         return InputValueDefinitionNode(
             description=description,
             name=name,
@@ -713,7 +774,7 @@ class Parser:
         self.expect_keyword("interface")
         name = self.parse_name()
         interfaces = self.parse_implements_interfaces()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         fields = self.parse_fields_definition()
         return InterfaceTypeDefinitionNode(
             description=description,
@@ -730,7 +791,7 @@ class Parser:
         description = self.parse_description()
         self.expect_keyword("union")
         name = self.parse_name()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         types = self.parse_union_member_types()
         return UnionTypeDefinitionNode(
             description=description,
@@ -742,16 +803,11 @@ class Parser:
 
     def parse_union_member_types(self) -> List[NamedTypeNode]:
         """UnionMemberTypes"""
-        types: List[NamedTypeNode] = []
-        if self.expect_optional_token(TokenKind.EQUALS):
-            # optional leading pipe
-            self.expect_optional_token(TokenKind.PIPE)
-            append = types.append
-            while True:
-                append(self.parse_named_type())
-                if not self.expect_optional_token(TokenKind.PIPE):
-                    break
-        return types
+        return (
+            self.delimited_many(TokenKind.PIPE, self.parse_named_type)
+            if self.expect_optional_token(TokenKind.EQUALS)
+            else []
+        )
 
     def parse_enum_type_definition(self) -> EnumTypeDefinitionNode:
         """UnionTypeDefinition"""
@@ -759,7 +815,7 @@ class Parser:
         description = self.parse_description()
         self.expect_keyword("enum")
         name = self.parse_name()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         values = self.parse_enum_values_definition()
         return EnumTypeDefinitionNode(
             description=description,
@@ -779,8 +835,8 @@ class Parser:
         """EnumValueDefinition: Description? EnumValue Directives[Const]?"""
         start = self._lexer.token
         description = self.parse_description()
-        name = self.parse_name()
-        directives = self.parse_directives(True)
+        name = self.parse_enum_value_name()
+        directives = self.parse_const_directives()
         return EnumValueDefinitionNode(
             description=description,
             name=name,
@@ -788,13 +844,24 @@ class Parser:
             loc=self.loc(start),
         )
 
+    def parse_enum_value_name(self) -> NameNode:
+        """EnumValue: Name but not ``true``, ``false`` or ``null``"""
+        if self._lexer.token.value in ("true", "false", "null"):
+            raise GraphQLSyntaxError(
+                self._lexer.source,
+                self._lexer.token.start,
+                f"{get_token_desc(self._lexer.token)} is reserved"
+                " and cannot be used for an enum value.",
+            )
+        return self.parse_name()
+
     def parse_input_object_type_definition(self) -> InputObjectTypeDefinitionNode:
         """InputObjectTypeDefinition"""
         start = self._lexer.token
         description = self.parse_description()
         self.expect_keyword("input")
         name = self.parse_name()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         fields = self.parse_input_fields_definition()
         return InputObjectTypeDefinitionNode(
             description=description,
@@ -815,7 +882,7 @@ class Parser:
         start = self._lexer.token
         self.expect_keyword("extend")
         self.expect_keyword("schema")
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         operation_types = self.optional_many(
             TokenKind.BRACE_L, self.parse_operation_type_definition, TokenKind.BRACE_R
         )
@@ -831,7 +898,7 @@ class Parser:
         self.expect_keyword("extend")
         self.expect_keyword("scalar")
         name = self.parse_name()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         if not directives:
             raise self.unexpected()
         return ScalarTypeExtensionNode(
@@ -845,7 +912,7 @@ class Parser:
         self.expect_keyword("type")
         name = self.parse_name()
         interfaces = self.parse_implements_interfaces()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         fields = self.parse_fields_definition()
         if not (interfaces or directives or fields):
             raise self.unexpected()
@@ -864,7 +931,7 @@ class Parser:
         self.expect_keyword("interface")
         name = self.parse_name()
         interfaces = self.parse_implements_interfaces()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         fields = self.parse_fields_definition()
         if not (interfaces or directives or fields):
             raise self.unexpected()
@@ -882,7 +949,7 @@ class Parser:
         self.expect_keyword("extend")
         self.expect_keyword("union")
         name = self.parse_name()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         types = self.parse_union_member_types()
         if not (directives or types):
             raise self.unexpected()
@@ -896,7 +963,7 @@ class Parser:
         self.expect_keyword("extend")
         self.expect_keyword("enum")
         name = self.parse_name()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         values = self.parse_enum_values_definition()
         if not (directives or values):
             raise self.unexpected()
@@ -910,7 +977,7 @@ class Parser:
         self.expect_keyword("extend")
         self.expect_keyword("input")
         name = self.parse_name()
-        directives = self.parse_directives(True)
+        directives = self.parse_const_directives()
         fields = self.parse_input_fields_definition()
         if not (directives or fields):
             raise self.unexpected()
@@ -940,15 +1007,7 @@ class Parser:
 
     def parse_directive_locations(self) -> List[NameNode]:
         """DirectiveLocations"""
-        # optional leading pipe
-        self.expect_optional_token(TokenKind.PIPE)
-        locations: List[NameNode] = []
-        append = locations.append
-        while True:
-            append(self.parse_directive_location())
-            if not self.expect_optional_token(TokenKind.PIPE):
-                break
-        return locations
+        return self.delimited_many(TokenKind.PIPE, self.parse_directive_location)
 
     def parse_directive_location(self) -> NameNode:
         """DirectiveLocation"""
@@ -978,12 +1037,12 @@ class Parser:
     def expect_token(self, kind: TokenKind) -> Token:
         """Expect the next token to be of the given kind.
 
-        If the next token is of the given kind, return that token after advancing
-        the lexer. Otherwise, do not change the parser state and throw an error.
+        If the next token is of the given kind, return that token after advancing the
+        lexer. Otherwise, do not change the parser state and throw an error.
         """
         token = self._lexer.token
         if token.kind == kind:
-            self._lexer.advance()
+            self.advance_lexer()
             return token
 
         raise GraphQLSyntaxError(
@@ -992,18 +1051,18 @@ class Parser:
             f"Expected {get_token_kind_desc(kind)}, found {get_token_desc(token)}.",
         )
 
-    def expect_optional_token(self, kind: TokenKind) -> Optional[Token]:
+    def expect_optional_token(self, kind: TokenKind) -> bool:
         """Expect the next token optionally to be of the given kind.
 
-        If the next token is of the given kind, return that token after advancing
-        the lexer. Otherwise, do not change the parser state and return None.
+        If the next token is of the given kind, return True after advancing the lexer.
+        Otherwise, do not change the parser state and return False.
         """
         token = self._lexer.token
         if token.kind == kind:
-            self._lexer.advance()
-            return token
+            self.advance_lexer()
+            return True
 
-        return None
+        return False
 
     def expect_keyword(self, value: str) -> None:
         """Expect the next token to be a given keyword.
@@ -1013,7 +1072,7 @@ class Parser:
         """
         token = self._lexer.token
         if token.kind == TokenKind.NAME and token.value == value:
-            self._lexer.advance()
+            self.advance_lexer()
         else:
             raise GraphQLSyntaxError(
                 self._lexer.source,
@@ -1029,7 +1088,7 @@ class Parser:
         """
         token = self._lexer.token
         if token.kind == TokenKind.NAME and token.value == value:
-            self._lexer.advance()
+            self.advance_lexer()
             return True
 
         return False
@@ -1054,7 +1113,8 @@ class Parser:
         self.expect_token(open_kind)
         nodes: List[T] = []
         append = nodes.append
-        while not self.expect_optional_token(close_kind):
+        expect_optional_token = partial(self.expect_optional_token, close_kind)
+        while not expect_optional_token():
             append(parse_fn())
         return nodes
 
@@ -1072,7 +1132,8 @@ class Parser:
         if self.expect_optional_token(open_kind):
             nodes = [parse_fn()]
             append = nodes.append
-            while not self.expect_optional_token(close_kind):
+            expect_optional_token = partial(self.expect_optional_token, close_kind)
+            while not expect_optional_token():
                 append(parse_fn())
             return nodes
         return []
@@ -1082,18 +1143,53 @@ class Parser:
     ) -> List[T]:
         """Fetch matching nodes, at least one.
 
-        Returns a non-empty list of parse nodes, determined by the ``parse_fn``.
-        This list begins with a lex token of ``open_kind`` and ends with a lex token of
+        Returns a non-empty list of parse nodes, determined by the ``parse_fn``. This
+        list begins with a lex token of ``open_kind`` and ends with a lex token of
         ``close_kind``. Advances the parser to the next lex token after the closing
         token.
         """
         self.expect_token(open_kind)
         nodes = [parse_fn()]
         append = nodes.append
-        while not self.expect_optional_token(close_kind):
+        expect_optional_token = partial(self.expect_optional_token, close_kind)
+        while not expect_optional_token():
             append(parse_fn())
         return nodes
 
+    def delimited_many(
+        self, delimiter_kind: TokenKind, parse_fn: Callable[[], T]
+    ) -> List[T]:
+        """Fetch many delimited nodes.
+
+        Returns a non-empty list of parse nodes, determined by the ``parse_fn``. This
+        list may begin with a lex token of ``delimiter_kind`` followed by items
+        separated by lex tokens of ``delimiter_kind``. Advances the parser to the next
+        lex token after the last item in the list.
+        """
+        expect_optional_token = partial(self.expect_optional_token, delimiter_kind)
+        expect_optional_token()
+        nodes: List[T] = []
+        append = nodes.append
+        while True:
+            append(parse_fn())
+            if not expect_optional_token():
+                break
+        return nodes
+
+    def advance_lexer(self) -> None:
+        max_tokens = self._max_tokens
+        token = self._lexer.advance()
+
+        if max_tokens is not None and token.kind != TokenKind.EOF:
+            self._token_counter += 1
+            if self._token_counter > max_tokens:
+                raise GraphQLSyntaxError(
+                    self._lexer.source,
+                    token.start,
+                    f"Document contains more than {max_tokens} tokens."
+                    " Parsing aborted.",
+                )
+
 
 def get_token_desc(token: Token) -> str:
     """Describe a token as a string for debugging."""
