@@ -1,21 +1,27 @@
 from operator import attrgetter, itemgetter
 from typing import (
     Any,
-    Callable,
+    Collection,
     Dict,
     List,
     Optional,
-    Sequence,
     Set,
     Tuple,
     Union,
     cast,
 )
 
-
 from ..error import GraphQLError
-from ..pyutils import inspect
-from ..language import NamedTypeNode, Node, OperationType, OperationTypeDefinitionNode
+from ..pyutils import Undefined, inspect
+from ..language import (
+    DirectiveNode,
+    InputValueDefinitionNode,
+    NamedTypeNode,
+    Node,
+    OperationType,
+    SchemaDefinitionNode,
+    SchemaExtensionNode,
+)
 from .definition import (
     GraphQLEnumType,
     GraphQLInputField,
@@ -33,10 +39,10 @@ from .definition import (
     is_output_type,
     is_union_type,
     is_required_argument,
+    is_required_input_field,
 )
-from ..utilities.assert_valid_name import is_valid_name_error
 from ..utilities.type_comparators import is_equal_type, is_type_sub_type_of
-from .directives import is_directive, GraphQLDirective
+from .directives import is_directive, GraphQLDeprecatedDirective
 from .introspection import is_introspection_type
 from .schema import GraphQLSchema, assert_schema
 
@@ -90,33 +96,30 @@ class SchemaValidationContext:
     errors: List[GraphQLError]
     schema: GraphQLSchema
 
-    def __init__(self, schema: GraphQLSchema) -> None:
+    def __init__(self, schema: GraphQLSchema):
         self.errors = []
         self.schema = schema
 
     def report_error(
         self,
         message: str,
-        nodes: Union[Optional[Node], Sequence[Optional[Node]]] = None,
-    ):
+        nodes: Union[Optional[Node], Collection[Optional[Node]]] = None,
+    ) -> None:
         if nodes and not isinstance(nodes, Node):
             nodes = [node for node in nodes if node]
-        nodes = cast(Optional[Sequence[Node]], nodes)
-        self.add_error(GraphQLError(message, nodes))
+        nodes = cast(Optional[Collection[Node]], nodes)
+        self.errors.append(GraphQLError(message, nodes))
 
-    def add_error(self, error: GraphQLError):
-        self.errors.append(error)
-
-    def validate_root_types(self):
+    def validate_root_types(self) -> None:
         schema = self.schema
-
         query_type = schema.query_type
         if not query_type:
             self.report_error("Query root type must be provided.", schema.ast_node)
         elif not is_object_type(query_type):
             self.report_error(
                 f"Query root type must be Object type, it cannot be {query_type}.",
-                get_operation_type_node(schema, query_type, OperationType.QUERY),
+                get_operation_type_node(schema, OperationType.QUERY)
+                or query_type.ast_node,
             )
 
         mutation_type = schema.mutation_type
@@ -124,7 +127,8 @@ class SchemaValidationContext:
             self.report_error(
                 "Mutation root type must be Object type if provided,"
                 f" it cannot be {mutation_type}.",
-                get_operation_type_node(schema, mutation_type, OperationType.MUTATION),
+                get_operation_type_node(schema, OperationType.MUTATION)
+                or mutation_type.ast_node,
             )
 
         subscription_type = schema.subscription_type
@@ -132,12 +136,11 @@ class SchemaValidationContext:
             self.report_error(
                 "Subscription root type must be Object type if provided,"
                 f" it cannot be {subscription_type}.",
-                get_operation_type_node(
-                    schema, subscription_type, OperationType.SUBSCRIPTION
-                ),
+                get_operation_type_node(schema, OperationType.SUBSCRIPTION)
+                or subscription_type.ast_node,
             )
 
-    def validate_directives(self):
+    def validate_directives(self) -> None:
         directives = self.schema.directives
         for directive in directives:
             # Ensure all directives are in fact GraphQL directives.
@@ -152,26 +155,10 @@ class SchemaValidationContext:
             self.validate_name(directive)
 
             # Ensure the arguments are valid.
-            arg_names: Set[str] = set()
             for arg_name, arg in directive.args.items():
                 # Ensure they are named correctly.
                 self.validate_name(arg, arg_name)
 
-                # Ensure they are unique per directive.
-                if arg_name in arg_names:
-                    self.report_error(
-                        f"Argument @{directive.name}({arg_name}:)"
-                        " can only be defined once.",
-                        directive.ast_node
-                        and [
-                            arg.ast_node
-                            for name, arg in directive.args.items()
-                            if name == arg_name
-                        ],
-                    )
-                    continue
-                arg_names.add(arg_name)
-
                 # Ensure the type is an input type.
                 if not is_input_type(arg.type):
                     self.report_error(
@@ -180,21 +167,34 @@ class SchemaValidationContext:
                         arg.ast_node,
                     )
 
-    def validate_name(self, node: Any, name: str = None):
+                if is_required_argument(arg) and arg.deprecation_reason is not None:
+                    self.report_error(
+                        f"Required argument @{directive.name}({arg_name}:)"
+                        " cannot be deprecated.",
+                        [
+                            get_deprecated_directive_node(arg.ast_node),
+                            arg.ast_node and arg.ast_node.type,
+                        ],
+                    )
+
+    def validate_name(self, node: Any, name: Optional[str] = None) -> None:
         # Ensure names are valid, however introspection types opt out.
         try:
             if not name:
                 name = node.name
             name = cast(str, name)
             ast_node = node.ast_node
-        except AttributeError:
+        except AttributeError:  # pragma: no cover
             pass
         else:
-            error = is_valid_name_error(name, ast_node)
-            if error:
-                self.add_error(error)
+            if name.startswith("__"):
+                self.report_error(
+                    f"Name {name!r} must not begin with '__',"
+                    " which is reserved by GraphQL introspection.",
+                    ast_node,
+                )
 
-    def validate_types(self):
+    def validate_types(self) -> None:
         validate_input_object_circular_refs = InputObjectCircularRefsValidator(self)
         for type_ in self.schema.type_map.values():
 
@@ -202,7 +202,7 @@ class SchemaValidationContext:
             if not is_named_type(type_):
                 self.report_error(
                     f"Expected GraphQL named type but got: {inspect(type_)}.",
-                    type_.ast_node if type_ else None,
+                    type_.ast_node if is_named_type(type_) else None,
                 )
                 continue
 
@@ -216,11 +216,14 @@ class SchemaValidationContext:
                 self.validate_fields(type_)
 
                 # Ensure objects implement the interfaces they claim to.
-                self.validate_object_interfaces(type_)
+                self.validate_interfaces(type_)
             elif is_interface_type(type_):
                 type_ = cast(GraphQLInterfaceType, type_)
                 # Ensure fields are valid.
                 self.validate_fields(type_)
+
+                # Ensure interfaces implement the interfaces they claim to.
+                self.validate_interfaces(type_)
             elif is_union_type(type_):
                 type_ = cast(GraphQLUnionType, type_)
                 # Ensure Unions include valid member types.
@@ -237,14 +240,16 @@ class SchemaValidationContext:
                 # Ensure Input Objects do not contain non-nullable circular references
                 validate_input_object_circular_refs(type_)
 
-    def validate_fields(self, type_: Union[GraphQLObjectType, GraphQLInterfaceType]):
+    def validate_fields(
+        self, type_: Union[GraphQLObjectType, GraphQLInterfaceType]
+    ) -> None:
         fields = type_.fields
 
         # Objects and Interfaces both must define one or more fields.
         if not fields:
             self.report_error(
                 f"Type {type_.name} must define one or more fields.",
-                get_all_nodes(type_),
+                [type_.ast_node, *type_.extension_ast_nodes],
             )
 
         for field_name, field in fields.items():
@@ -261,174 +266,207 @@ class SchemaValidationContext:
                 )
 
             # Ensure the arguments are valid.
-            arg_names: Set[str] = set()
             for arg_name, arg in field.args.items():
                 # Ensure they are named correctly.
                 self.validate_name(arg, arg_name)
 
-                # Ensure they are unique per field.
-                if arg_name in arg_names:
-                    self.report_error(
-                        "Field argument"
-                        f" {type_.name}.{field_name}({arg_name}:)"
-                        " can only be defined once.",
-                        [
-                            arg.ast_node
-                            for name, arg in field.args.items()
-                            if name == arg_name
-                        ],
-                    )
-                    break
-                arg_names.add(arg_name)
-
                 # Ensure the type is an input type.
                 if not is_input_type(arg.type):
                     self.report_error(
-                        "Field argument"
-                        f" {type_.name}.{field_name}({arg_name}:)"
+                        f"The type of {type_.name}.{field_name}({arg_name}:)"
                         f" must be Input Type but got: {inspect(arg.type)}.",
                         arg.ast_node and arg.ast_node.type,
                     )
 
-    def validate_object_interfaces(self, obj: GraphQLObjectType):
-        implemented_type_names: Set[str] = set()
-        for iface in obj.interfaces:
+                if is_required_argument(arg) and arg.deprecation_reason is not None:
+                    self.report_error(
+                        f"Required argument {type_.name}.{field_name}({arg_name}:)"
+                        " cannot be deprecated.",
+                        [
+                            get_deprecated_directive_node(arg.ast_node),
+                            arg.ast_node and arg.ast_node.type,
+                        ],
+                    )
+
+    def validate_interfaces(
+        self, type_: Union[GraphQLObjectType, GraphQLInterfaceType]
+    ) -> None:
+        iface_type_names: Set[str] = set()
+        for iface in type_.interfaces:
             if not is_interface_type(iface):
                 self.report_error(
-                    f"Type {obj.name} must only implement Interface"
+                    f"Type {type_.name} must only implement Interface"
                     f" types, it cannot implement {inspect(iface)}.",
-                    get_all_implements_interface_nodes(obj, iface),
+                    get_all_implements_interface_nodes(type_, iface),
                 )
                 continue
-            if iface.name in implemented_type_names:
-                self.report_error(
-                    f"Type {obj.name} can only implement {iface.name} once.",
-                    get_all_implements_interface_nodes(obj, iface),
-                )
-                continue
-            implemented_type_names.add(iface.name)
-            self.validate_object_implements_interface(obj, iface)
 
-    def validate_object_implements_interface(
-        self, obj: GraphQLObjectType, iface: GraphQLInterfaceType
-    ):
-        obj_fields, iface_fields = obj.fields, iface.fields
+            if type_ is iface:
+                self.report_error(
+                    f"Type {type_.name} cannot implement itself"
+                    " because it would create a circular reference.",
+                    get_all_implements_interface_nodes(type_, iface),
+                )
+
+            if iface.name in iface_type_names:
+                self.report_error(
+                    f"Type {type_.name} can only implement {iface.name} once.",
+                    get_all_implements_interface_nodes(type_, iface),
+                )
+                continue
+
+            iface_type_names.add(iface.name)
+
+            self.validate_type_implements_ancestors(type_, iface)
+            self.validate_type_implements_interface(type_, iface)
+
+    def validate_type_implements_interface(
+        self,
+        type_: Union[GraphQLObjectType, GraphQLInterfaceType],
+        iface: GraphQLInterfaceType,
+    ) -> None:
+        type_fields, iface_fields = type_.fields, iface.fields
 
         # Assert each interface field is implemented.
         for field_name, iface_field in iface_fields.items():
-            obj_field = obj_fields.get(field_name)
+            type_field = type_fields.get(field_name)
 
             # Assert interface field exists on object.
-            if not obj_field:
+            if not type_field:
                 self.report_error(
                     f"Interface field {iface.name}.{field_name}"
-                    f" expected but {obj.name} does not provide it.",
-                    [iface_field.ast_node, *get_all_nodes(obj)],
+                    f" expected but {type_.name} does not provide it.",
+                    [
+                        iface_field.ast_node,
+                        type_.ast_node,
+                        *type_.extension_ast_nodes,
+                    ],
                 )
                 continue
 
-            # Assert interface field type is satisfied by object field type, by being
+            # Assert interface field type is satisfied by type field type, by being
             # a valid subtype (covariant).
-            if not is_type_sub_type_of(self.schema, obj_field.type, iface_field.type):
+            if not is_type_sub_type_of(self.schema, type_field.type, iface_field.type):
                 self.report_error(
                     f"Interface field {iface.name}.{field_name}"
                     f" expects type {iface_field.type}"
-                    f" but {obj.name}.{field_name}"
-                    f" is type {obj_field.type}.",
+                    f" but {type_.name}.{field_name}"
+                    f" is type {type_field.type}.",
                     [
                         iface_field.ast_node and iface_field.ast_node.type,
-                        obj_field.ast_node and obj_field.ast_node.type,
+                        type_field.ast_node and type_field.ast_node.type,
                     ],
                 )
 
             # Assert each interface field arg is implemented.
             for arg_name, iface_arg in iface_field.args.items():
-                obj_arg = obj_field.args.get(arg_name)
+                type_arg = type_field.args.get(arg_name)
 
                 # Assert interface field arg exists on object field.
-                if not obj_arg:
+                if not type_arg:
                     self.report_error(
                         "Interface field argument"
                         f" {iface.name}.{field_name}({arg_name}:)"
-                        f" expected but {obj.name}.{field_name}"
+                        f" expected but {type_.name}.{field_name}"
                         " does not provide it.",
-                        [iface_arg.ast_node, obj_field.ast_node],
+                        [iface_arg.ast_node, type_field.ast_node],
                     )
                     continue
 
                 # Assert interface field arg type matches object field arg type
                 # (invariant).
-                if not is_equal_type(iface_arg.type, obj_arg.type):
+                if not is_equal_type(iface_arg.type, type_arg.type):
                     self.report_error(
                         "Interface field argument"
                         f" {iface.name}.{field_name}({arg_name}:)"
                         f" expects type {iface_arg.type}"
-                        f" but {obj.name}.{field_name}({arg_name}:)"
-                        f" is type {obj_arg.type}.",
+                        f" but {type_.name}.{field_name}({arg_name}:)"
+                        f" is type {type_arg.type}.",
                         [
                             iface_arg.ast_node and iface_arg.ast_node.type,
-                            obj_arg.ast_node and obj_arg.ast_node.type,
+                            type_arg.ast_node and type_arg.ast_node.type,
                         ],
                     )
 
             # Assert additional arguments must not be required.
-            for arg_name, obj_arg in obj_field.args.items():
+            for arg_name, type_arg in type_field.args.items():
                 iface_arg = iface_field.args.get(arg_name)
-                if not iface_arg and is_required_argument(obj_arg):
+                if not iface_arg and is_required_argument(type_arg):
                     self.report_error(
-                        f"Object field {obj.name}.{field_name} includes"
+                        f"Object field {type_.name}.{field_name} includes"
                         f" required argument {arg_name} that is missing from"
                         f" the Interface field {iface.name}.{field_name}.",
-                        [obj_arg.ast_node, iface_field.ast_node],
+                        [type_arg.ast_node, iface_field.ast_node],
                     )
 
-    def validate_union_members(self, union: GraphQLUnionType):
+    def validate_type_implements_ancestors(
+        self,
+        type_: Union[GraphQLObjectType, GraphQLInterfaceType],
+        iface: GraphQLInterfaceType,
+    ) -> None:
+        type_interfaces, iface_interfaces = type_.interfaces, iface.interfaces
+        for transitive in iface_interfaces:
+            if transitive not in type_interfaces:
+                self.report_error(
+                    (
+                        f"Type {type_.name} cannot implement {iface.name}"
+                        " because it would create a circular reference."
+                        if transitive is type_
+                        else f"Type {type_.name} must implement {transitive.name}"
+                        f" because it is implemented by {iface.name}."
+                    ),
+                    get_all_implements_interface_nodes(iface, transitive)
+                    + get_all_implements_interface_nodes(type_, iface),
+                )
+
+    def validate_union_members(self, union: GraphQLUnionType) -> None:
         member_types = union.types
 
         if not member_types:
             self.report_error(
                 f"Union type {union.name} must define one or more member types.",
-                get_all_nodes(union),
+                [union.ast_node, *union.extension_ast_nodes],
             )
 
         included_type_names: Set[str] = set()
         for member_type in member_types:
-            if member_type.name in included_type_names:
+            if is_object_type(member_type):
+                if member_type.name in included_type_names:
+                    self.report_error(
+                        f"Union type {union.name} can only include type"
+                        f" {member_type.name} once.",
+                        get_union_member_type_nodes(union, member_type.name),
+                    )
+                else:
+                    included_type_names.add(member_type.name)
+            else:
                 self.report_error(
-                    f"Union type {union.name} can only include type"
-                    f" {member_type.name} once.",
-                    get_union_member_type_nodes(union, member_type.name),
+                    f"Union type {union.name} can only include Object types,"
+                    f" it cannot include {inspect(member_type)}.",
+                    get_union_member_type_nodes(union, str(member_type)),
                 )
-                continue
-            included_type_names.add(member_type.name)
 
-    def validate_enum_values(self, enum_type: GraphQLEnumType):
+    def validate_enum_values(self, enum_type: GraphQLEnumType) -> None:
         enum_values = enum_type.values
 
         if not enum_values:
             self.report_error(
                 f"Enum type {enum_type.name} must define one or more values.",
-                get_all_nodes(enum_type),
+                [enum_type.ast_node, *enum_type.extension_ast_nodes],
             )
 
         for value_name, enum_value in enum_values.items():
             # Ensure valid name.
             self.validate_name(enum_value, value_name)
-            if value_name in ("true", "false", "null"):
-                self.report_error(
-                    f"Enum type {enum_type.name} cannot include value:"
-                    f" {value_name}.",
-                    enum_value.ast_node,
-                )
 
-    def validate_input_fields(self, input_obj: GraphQLInputObjectType):
+    def validate_input_fields(self, input_obj: GraphQLInputObjectType) -> None:
         fields = input_obj.fields
 
         if not fields:
             self.report_error(
                 f"Input Object type {input_obj.name}"
                 " must define one or more fields.",
-                get_all_nodes(input_obj),
+                [input_obj.ast_node, *input_obj.extension_ast_nodes],
             )
 
         # Ensure the arguments are valid
@@ -445,18 +483,51 @@ class SchemaValidationContext:
                     field.ast_node.type if field.ast_node else None,
                 )
 
+            if is_required_input_field(field) and field.deprecation_reason is not None:
+                self.report_error(
+                    f"Required input field {input_obj.name}.{field_name}"
+                    " cannot be deprecated.",
+                    [
+                        get_deprecated_directive_node(field.ast_node),
+                        field.ast_node and field.ast_node.type,
+                    ],
+                )
+
+            if input_obj.is_one_of:
+                self.validate_one_of_input_object_field(input_obj, field_name, field)
+
+    def validate_one_of_input_object_field(
+        self,
+        type_: GraphQLInputObjectType,
+        field_name: str,
+        field: GraphQLInputField,
+    ) -> None:
+        if is_non_null_type(field.type):
+            self.report_error(
+                f"OneOf input field {type_.name}.{field_name} must be nullable.",
+                field.ast_node and field.ast_node.type,
+            )
+
+        if field.default_value is not Undefined:
+            self.report_error(
+                f"OneOf input field {type_.name}.{field_name}"
+                " cannot have a default value.",
+                field.ast_node,
+            )
+
 
 def get_operation_type_node(
-    schema: GraphQLSchema, type_: GraphQLObjectType, operation: OperationType
+    schema: GraphQLSchema, operation: OperationType
 ) -> Optional[Node]:
-    operation_nodes = cast(
-        List[OperationTypeDefinitionNode],
-        get_all_sub_nodes(schema, attrgetter("operation_types")),
-    )
-    for node in operation_nodes:
-        if node.operation == operation:
-            return node.type
-    return type_.ast_node
+    ast_node: Optional[Union[SchemaDefinitionNode, SchemaExtensionNode]]
+    for ast_node in [schema.ast_node, *(schema.extension_ast_nodes or ())]:
+        if ast_node:
+            operation_types = ast_node.operation_types
+            if operation_types:  # pragma: no cover else
+                for operation_type in operation_types:
+                    if operation_type.operation == operation:
+                        return operation_type.type
+    return None
 
 
 class InputObjectCircularRefsValidator:
@@ -472,7 +543,7 @@ class InputObjectCircularRefsValidator:
         # Position in the type path
         self.field_path_index_by_type_name: Dict[str, int] = {}
 
-    def __call__(self, input_obj: GraphQLInputObjectType):
+    def __call__(self, input_obj: GraphQLInputObjectType) -> None:
         """Detect cycles recursively."""
         # This does a straight-forward DFS to find cycles.
         # It does not terminate when a cycle was found but continues to explore
@@ -502,7 +573,7 @@ class InputObjectCircularRefsValidator:
                         " within itself through a series of non-null fields:"
                         f" '{'.'.join(field_names)}'.",
                         cast(
-                            Sequence[Node],
+                            Collection[Node],
                             map(attrgetter("ast_node"), map(itemgetter(1), cycle_path)),
                         ),
                     )
@@ -511,56 +582,52 @@ class InputObjectCircularRefsValidator:
         del self.field_path_index_by_type_name[name]
 
 
-SDLDefinedObject = Union[
-    GraphQLSchema,
-    GraphQLDirective,
-    GraphQLInterfaceType,
-    GraphQLObjectType,
-    GraphQLInputObjectType,
-    GraphQLUnionType,
-    GraphQLEnumType,
-]
-
-
-def get_all_nodes(obj: SDLDefinedObject) -> List[Node]:
-    node = obj.ast_node
-    nodes: List[Node] = [node] if node else []
-    extension_nodes = getattr(obj, "extension_ast_nodes", None)
-    if extension_nodes:
-        nodes.extend(extension_nodes)
-    return nodes
-
-
-def get_all_sub_nodes(
-    obj: SDLDefinedObject, getter: Callable[[Node], List[Node]]
-) -> List[Node]:
-    result: List[Node] = []
-    for ast_node in get_all_nodes(obj):
-        sub_nodes = getter(ast_node)
-        if sub_nodes:
-            result.extend(sub_nodes)
-    return result
-
-
 def get_all_implements_interface_nodes(
-    type_: GraphQLObjectType, iface: GraphQLInterfaceType
+    type_: Union[GraphQLObjectType, GraphQLInterfaceType], iface: GraphQLInterfaceType
 ) -> List[NamedTypeNode]:
-    implements_nodes = cast(
-        List[NamedTypeNode], get_all_sub_nodes(type_, attrgetter("interfaces"))
-    )
-    return [
-        iface_node
-        for iface_node in implements_nodes
-        if iface_node.name.value == iface.name
-    ]
+    ast_node = type_.ast_node
+    nodes = type_.extension_ast_nodes
+    if ast_node is not None:
+        nodes = [ast_node, *nodes]  # type: ignore
+    implements_nodes: List[NamedTypeNode] = []
+    for node in nodes:
+        iface_nodes = node.interfaces
+        if iface_nodes:  # pragma: no cover else
+            implements_nodes.extend(
+                iface_node
+                for iface_node in iface_nodes
+                if iface_node.name.value == iface.name
+            )
+    return implements_nodes
 
 
 def get_union_member_type_nodes(
     union: GraphQLUnionType, type_name: str
-) -> Optional[List[NamedTypeNode]]:
-    union_nodes = cast(
-        List[NamedTypeNode], get_all_sub_nodes(union, attrgetter("types"))
-    )
-    return [
-        union_node for union_node in union_nodes if union_node.name.value == type_name
-    ]
+) -> List[NamedTypeNode]:
+    ast_node = union.ast_node
+    nodes = union.extension_ast_nodes
+    if ast_node is not None:
+        nodes = [ast_node, *nodes]  # type: ignore
+    member_type_nodes: List[NamedTypeNode] = []
+    for node in nodes:
+        type_nodes = node.types
+        if type_nodes:  # pragma: no cover else
+            member_type_nodes.extend(
+                type_node
+                for type_node in type_nodes
+                if type_node.name.value == type_name
+            )
+    return member_type_nodes
+
+
+def get_deprecated_directive_node(
+    definition_node: Optional[Union[InputValueDefinitionNode]],
+) -> Optional[DirectiveNode]:
+    directives = definition_node and definition_node.directives
+    if directives:
+        for directive in directives:
+            if (
+                directive.name.value == GraphQLDeprecatedDirective.name
+            ):  # pragma: no cover else
+                return directive
+    return None  # pragma: no cover
