@@ -1,17 +1,13 @@
-from typing import Union
+from typing import cast, Union
 
-from ..language import (
-    DocumentNode,
-    Source,
-    parse,
-)
+from ..language import DocumentNode, Source, parse
 from ..type import (
-    GraphQLDeprecatedDirective,
-    GraphQLIncludeDirective,
+    GraphQLObjectType,
     GraphQLSchema,
-    GraphQLSkipDirective,
+    GraphQLSchemaKwargs,
+    specified_directives,
 )
-from .extend_schema import extend_schema_impl
+from .extend_schema import ExtendSchemaImpl
 
 __all__ = [
     "build_ast_schema",
@@ -29,16 +25,16 @@ def build_ast_schema(
     This takes the ast of a schema document produced by the parse function in
     src/language/parser.py.
 
-    If no schema definition is provided, then it will look for types named Query
-    and Mutation.
+    If no schema definition is provided, then it will look for types named Query,
+    Mutation and Subscription.
 
     Given that AST it constructs a GraphQLSchema. The resulting schema has no
     resolve methods, so execution will use default resolvers.
 
     When building a schema from a GraphQL service's introspection result, it might
-    be safe to assume the schema is valid. Set `assume_valid` to True to assume the
-    produced schema is valid. Set `assume_valid_sdl` to True to assume it is already
-    a valid SDL document.
+    be safe to assume the schema is valid. Set ``assume_valid`` to ``True`` to assume
+    the produced schema is valid. Set ``assume_valid_sdl`` to ``True`` to assume it is
+    already a valid SDL document.
     """
     if not isinstance(document_ast, DocumentNode):
         raise TypeError("Must provide valid Document AST.")
@@ -48,49 +44,61 @@ def build_ast_schema(
 
         assert_valid_sdl(document_ast)
 
-    schema_kwargs = extend_schema_impl(empty_schema_config, document_ast, assume_valid)
+    empty_schema_kwargs = GraphQLSchemaKwargs(
+        query=None,
+        mutation=None,
+        subscription=None,
+        description=None,
+        types=(),
+        directives=(),
+        extensions={},
+        ast_node=None,
+        extension_ast_nodes=(),
+        assume_valid=False,
+    )
+    schema_kwargs = ExtendSchemaImpl.extend_schema_args(
+        empty_schema_kwargs, document_ast, assume_valid
+    )
 
     if not schema_kwargs["ast_node"]:
-        for type_ in schema_kwargs["types"]:
+        for type_ in schema_kwargs["types"] or ():
             # Note: While this could make early assertions to get the correctly
             # typed values below, that would throw immediately while type system
             # validation with validate_schema() will produce more actionable results.
             type_name = type_.name
             if type_name == "Query":
-                schema_kwargs["query"] = type_
+                schema_kwargs["query"] = cast(GraphQLObjectType, type_)
             elif type_name == "Mutation":
-                schema_kwargs["mutation"] = type_
+                schema_kwargs["mutation"] = cast(GraphQLObjectType, type_)
             elif type_name == "Subscription":
-                schema_kwargs["subscription"] = type_
+                schema_kwargs["subscription"] = cast(GraphQLObjectType, type_)
 
-    directives = schema_kwargs["directives"]
     # If specified directives were not explicitly declared, add them.
-    if not any(directive.name == "skip" for directive in directives):
-        directives.append(GraphQLSkipDirective)
-    if not any(directive.name == "include" for directive in directives):
-        directives.append(GraphQLIncludeDirective)
-    if not any(directive.name == "deprecated" for directive in directives):
-        directives.append(GraphQLDeprecatedDirective)
+    directives = schema_kwargs["directives"]
+    directive_names = set(directive.name for directive in directives)
+    missing_directives = []
+    for directive in specified_directives:
+        if directive.name not in directive_names:
+            missing_directives.append(directive)
+    if missing_directives:
+        schema_kwargs["directives"] = directives + tuple(missing_directives)
 
     return GraphQLSchema(**schema_kwargs)
 
 
-empty_schema_config = GraphQLSchema(directives=[]).to_kwargs()
-
-
 def build_schema(
     source: Union[str, Source],
-    assume_valid=False,
-    assume_valid_sdl=False,
-    no_location=False,
-    experimental_fragment_variables=False,
+    assume_valid: bool = False,
+    assume_valid_sdl: bool = False,
+    no_location: bool = False,
+    allow_legacy_fragment_variables: bool = False,
 ) -> GraphQLSchema:
     """Build a GraphQLSchema directly from a source document."""
     return build_ast_schema(
         parse(
             source,
             no_location=no_location,
-            experimental_fragment_variables=experimental_fragment_variables,
+            allow_legacy_fragment_variables=allow_legacy_fragment_variables,
         ),
         assume_valid=assume_valid,
         assume_valid_sdl=assume_valid_sdl,
