@@ -1,10 +1,8 @@
 from enum import Enum
-from operator import attrgetter
-from typing import Any, Dict, List, NamedTuple, Union, cast
+from typing import Any, Collection, Dict, List, NamedTuple, Union, cast
 
-from ..error import INVALID
-from ..language import print_ast, visit, Visitor
-from ..pyutils import inspect
+from ..language import print_ast
+from ..pyutils import inspect, Undefined
 from ..type import (
     GraphQLEnumType,
     GraphQLField,
@@ -22,13 +20,15 @@ from ..type import (
     is_interface_type,
     is_list_type,
     is_named_type,
-    is_required_argument,
-    is_required_input_field,
     is_non_null_type,
     is_object_type,
+    is_required_argument,
+    is_required_input_field,
     is_scalar_type,
+    is_specified_scalar_type,
     is_union_type,
 )
+from ..utilities.sort_value_node import sort_value_node
 from .ast_from_value import ast_from_value
 
 __all__ = [
@@ -47,7 +47,7 @@ class BreakingChangeType(Enum):
     TYPE_REMOVED_FROM_UNION = 20
     VALUE_REMOVED_FROM_ENUM = 21
     REQUIRED_INPUT_FIELD_ADDED = 22
-    INTERFACE_REMOVED_FROM_OBJECT = 23
+    IMPLEMENTED_INTERFACE_REMOVED = 23
     FIELD_REMOVED = 30
     FIELD_CHANGED_KIND = 31
     REQUIRED_ARG_ADDED = 40
@@ -56,7 +56,8 @@ class BreakingChangeType(Enum):
     DIRECTIVE_REMOVED = 50
     DIRECTIVE_ARG_REMOVED = 51
     REQUIRED_DIRECTIVE_ARG_ADDED = 52
-    DIRECTIVE_LOCATION_REMOVED = 53
+    DIRECTIVE_REPEATABLE_REMOVED = 53
+    DIRECTIVE_LOCATION_REMOVED = 54
 
 
 class DangerousChangeType(Enum):
@@ -64,7 +65,7 @@ class DangerousChangeType(Enum):
     TYPE_ADDED_TO_UNION = 61
     OPTIONAL_INPUT_FIELD_ADDED = 62
     OPTIONAL_ARG_ADDED = 63
-    INTERFACE_ADDED_TO_OBJECT = 64
+    IMPLEMENTED_INTERFACE_ADDED = 64
     ARG_DEFAULT_VALUE_CHANGE = 65
 
 
@@ -133,7 +134,7 @@ def find_directive_changes(
             )
         )
 
-    for (old_directive, new_directive) in directives_diff.persisted:
+    for old_directive, new_directive in directives_diff.persisted:
         args_diff = dict_diff(old_directive.args, new_directive.args)
 
         for arg_name, new_arg in args_diff.added.items():
@@ -154,6 +155,14 @@ def find_directive_changes(
                 )
             )
 
+        if old_directive.is_repeatable and not new_directive.is_repeatable:
+            schema_changes.append(
+                BreakingChange(
+                    BreakingChangeType.DIRECTIVE_REPEATABLE_REMOVED,
+                    f"Repeatable flag was removed from {old_directive.name}.",
+                )
+            )
+
         for location in old_directive.locations:
             if location not in new_directive.locations:
                 schema_changes.append(
@@ -172,9 +181,17 @@ def find_type_changes(
     schema_changes: List[Change] = []
     types_diff = dict_diff(old_schema.type_map, new_schema.type_map)
 
-    for type_name in types_diff.removed:
+    for type_name, old_type in types_diff.removed.items():
         schema_changes.append(
-            BreakingChange(BreakingChangeType.TYPE_REMOVED, f"{type_name} was removed.")
+            BreakingChange(
+                BreakingChangeType.TYPE_REMOVED,
+                (
+                    f"Standard scalar {type_name} was removed"
+                    " because it is not referenced anymore."
+                    if is_specified_scalar_type(old_type)
+                    else f"{type_name} was removed."
+                ),
+            )
         )
 
     for type_name, (old_type, new_type) in types_diff.persisted.items():
@@ -185,9 +202,15 @@ def find_type_changes(
         elif is_input_object_type(old_type) and is_input_object_type(new_type):
             schema_changes.extend(find_input_object_type_changes(old_type, new_type))
         elif is_object_type(old_type) and is_object_type(new_type):
-            schema_changes.extend(find_object_type_changes(old_type, new_type))
+            schema_changes.extend(find_field_changes(old_type, new_type))
+            schema_changes.extend(
+                find_implemented_interfaces_changes(old_type, new_type)
+            )
         elif is_interface_type(old_type) and is_interface_type(new_type):
             schema_changes.extend(find_field_changes(old_type, new_type))
+            schema_changes.extend(
+                find_implemented_interfaces_changes(old_type, new_type)
+            )
         elif old_type.__class__ is not new_type.__class__:
             schema_changes.append(
                 BreakingChange(
@@ -299,16 +322,17 @@ def find_enum_type_changes(
     return schema_changes
 
 
-def find_object_type_changes(
-    old_type: GraphQLObjectType, new_type: GraphQLObjectType
+def find_implemented_interfaces_changes(
+    old_type: Union[GraphQLObjectType, GraphQLInterfaceType],
+    new_type: Union[GraphQLObjectType, GraphQLInterfaceType],
 ) -> List[Change]:
-    schema_changes: List[Change] = find_field_changes(old_type, new_type)
+    schema_changes: List[Change] = []
     interfaces_diff = list_diff(old_type.interfaces, new_type.interfaces)
 
     for interface in interfaces_diff.added:
         schema_changes.append(
             DangerousChange(
-                DangerousChangeType.INTERFACE_ADDED_TO_OBJECT,
+                DangerousChangeType.IMPLEMENTED_INTERFACE_ADDED,
                 f"{interface.name} added to interfaces implemented by {old_type.name}.",
             )
         )
@@ -316,7 +340,7 @@ def find_object_type_changes(
     for interface in interfaces_diff.removed:
         schema_changes.append(
             BreakingChange(
-                BreakingChangeType.INTERFACE_REMOVED_FROM_OBJECT,
+                BreakingChangeType.IMPLEMENTED_INTERFACE_REMOVED,
                 f"{old_type.name} no longer implements interface {interface.name}.",
             )
         )
@@ -388,8 +412,8 @@ def find_arg_changes(
                     f" {old_arg.type} to {new_arg.type}.",
                 )
             )
-        elif old_arg.default_value is not INVALID:
-            if new_arg.default_value is INVALID:
+        elif old_arg.default_value is not Undefined:
+            if new_arg.default_value is Undefined:
                 schema_changes.append(
                     DangerousChange(
                         DangerousChangeType.ARG_DEFAULT_VALUE_CHANGE,
@@ -527,25 +551,14 @@ def type_kind_name(type_: GraphQLNamedType) -> str:
         return "an Input type"
 
     # Not reachable. All possible output types have been considered.
-    raise TypeError(f"Unexpected type {inspect(type)}")  # pragma: no cover
+    raise TypeError(f"Unexpected type {inspect(type)}")
 
 
 def stringify_value(value: Any, type_: GraphQLInputType) -> str:
     ast = ast_from_value(value, type_)
-    if ast is None:
+    if ast is None:  # pragma: no cover
         raise TypeError(f"Invalid value: {inspect(value)}")
-
-    # noinspection PyMethodMayBeStatic
-    class SortVisitor(Visitor):
-        def enter_object_value(self, object_node, *_args):
-            object_node.fields = sorted(
-                object_node.fields, key=attrgetter("name.value")
-            )
-            return object_node
-
-    sorted_ast = visit(ast, SortVisitor())
-
-    return print_ast(sorted_ast)
+    return print_ast(sort_value_node(ast))
 
 
 class ListDiff(NamedTuple):
@@ -556,7 +569,7 @@ class ListDiff(NamedTuple):
     persisted: List
 
 
-def list_diff(old_list: List, new_list: List) -> ListDiff:
+def list_diff(old_list: Collection, new_list: Collection) -> ListDiff:
     """Get differences between two lists of named items."""
     added = []
     persisted = []
