@@ -4,9 +4,9 @@ from typing import Any, Dict, List, Optional, Union
 from ..language import DirectiveLocation
 
 try:
-    from typing import TypedDict, Literal
+    from typing import Literal, TypedDict
 except ImportError:  # Python < 3.8
-    from typing_extensions import TypedDict, Literal  # type: ignore
+    from typing_extensions import Literal, TypedDict
 
 __all__ = [
     "get_introspection_query",
@@ -34,6 +34,7 @@ def get_introspection_query(
     directive_is_repeatable: bool = False,
     schema_description: bool = False,
     input_value_deprecation: bool = False,
+    input_object_one_of: bool = False,
 ) -> str:
     """Get a query for introspection.
 
@@ -45,6 +46,7 @@ def get_introspection_query(
     maybe_specified_by_url = "specifiedByURL" if specified_by_url else ""
     maybe_directive_is_repeatable = "isRepeatable" if directive_is_repeatable else ""
     maybe_schema_description = maybe_description if schema_description else ""
+    maybe_input_object_one_of = "isOneOf" if input_object_one_of else ""
 
     def input_deprecation(string: str) -> Optional[str]:
         return string if input_value_deprecation else ""
@@ -77,6 +79,7 @@ def get_introspection_query(
           name
           {maybe_description}
           {maybe_specified_by_url}
+          {maybe_input_object_one_of}
           fields(includeDeprecated: true) {{
             name
             {maybe_description}
@@ -139,6 +142,14 @@ def get_introspection_query(
                       ofType {{
                         kind
                         name
+                        ofType {{
+                          kind
+                          name
+                          ofType {{
+                            kind
+                            name
+                          }}
+                        }}
                       }}
                     }}
                   }}
@@ -235,6 +246,7 @@ class IntrospectionEnumType(WithName):
 class IntrospectionInputObjectType(WithName):
     kind: Literal["input_object"]
     inputFields: List[IntrospectionInputValue]
+    isOneOf: bool
 
 
 IntrospectionType = Union[
@@ -246,7 +258,6 @@ IntrospectionType = Union[
     IntrospectionInputObjectType,
 ]
 
-
 IntrospectionOutputType = Union[
     IntrospectionScalarType,
     IntrospectionObjectType,
@@ -255,7 +266,6 @@ IntrospectionOutputType = Union[
     IntrospectionEnumType,
 ]
 
-
 IntrospectionInputType = Union[
     IntrospectionScalarType, IntrospectionEnumType, IntrospectionInputObjectType
 ]
@@ -284,7 +294,9 @@ class IntrospectionSchema(MaybeWithDescription):
     directives: List[IntrospectionDirective]
 
 
-class IntrospectionQuery(TypedDict):
-    """The root typed dictionary for schema introspections."""
-
-    __schema: IntrospectionSchema
+# The root typed dictionary for schema introspections.
+# Note: We don't use class syntax here since the key looks like a private attribute.
+IntrospectionQuery = TypedDict(
+    "IntrospectionQuery",
+    {"__schema": IntrospectionSchema},
+)
