@@ -1,23 +1,22 @@
-from typing import Any, Dict
+from typing import cast
 
 from ..error import GraphQLError
 from ..language import parse
 from ..type import GraphQLSchema
-from .get_introspection_query import get_introspection_query
+from .get_introspection_query import get_introspection_query, IntrospectionQuery
 
 __all__ = ["introspection_from_schema"]
 
 
-IntrospectionSchema = Dict[str, Any]
-
-
 def introspection_from_schema(
     schema: GraphQLSchema,
     descriptions: bool = True,
-    specified_by_url: bool = False,
+    specified_by_url: bool = True,
     directive_is_repeatable: bool = True,
     schema_description: bool = True,
-) -> IntrospectionSchema:
+    input_value_deprecation: bool = True,
+    input_object_one_of: bool = True,
+) -> IntrospectionQuery:
     """Build an IntrospectionQuery from a GraphQLSchema
 
     IntrospectionQuery is useful for utilities that care about type and field
@@ -28,7 +27,12 @@ def introspection_from_schema(
     """
     document = parse(
         get_introspection_query(
-            descriptions, specified_by_url, directive_is_repeatable, schema_description
+            descriptions,
+            specified_by_url,
+            directive_is_repeatable,
+            schema_description,
+            input_value_deprecation,
+            input_object_one_of,
         )
     )
 
@@ -41,4 +45,4 @@ def introspection_from_schema(
         raise result.errors[0]
     if not result.data:  # pragma: no cover
         raise GraphQLError("Introspection did not return a result")
-    return result.data
+    return cast(IntrospectionQuery, result.data)
