@@ -1,4 +1,4 @@
-from typing import cast, Dict, List, Union
+from typing import cast, Any, Dict, List, Optional, Tuple, Union
 
 from ...error import GraphQLError
 from ...language import (
@@ -11,33 +11,23 @@ from ...language import (
 from ...type import specified_directives
 from . import ASTValidationRule, SDLValidationContext, ValidationContext
 
-__all__ = [
-    "KnownDirectivesRule",
-    "unknown_directive_message",
-    "misplaced_directive_message",
-]
-
-
-def unknown_directive_message(directive_name: str) -> str:
-    return f"Unknown directive '{directive_name}'."
-
-
-def misplaced_directive_message(directive_name: str, location: str) -> str:
-    return f"Directive '{directive_name}' may not be used on {location}."
+__all__ = ["KnownDirectivesRule"]
 
 
 class KnownDirectivesRule(ASTValidationRule):
     """Known directives
 
-    A GraphQL document is only valid if all `@directives` are known by the schema and
+    A GraphQL document is only valid if all ``@directives`` are known by the schema and
     legally positioned.
+
+    See https://spec.graphql.org/draft/#sec-Directives-Are-Defined
     """
 
     context: Union[ValidationContext, SDLValidationContext]
 
-    def __init__(self, context: Union[ValidationContext, SDLValidationContext]) -> None:
+    def __init__(self, context: Union[ValidationContext, SDLValidationContext]):
         super().__init__(context)
-        locations_map: Dict[str, List[DirectiveLocation]] = {}
+        locations_map: Dict[str, Tuple[DirectiveLocation, ...]] = {}
 
         schema = context.schema
         defined_directives = (
@@ -48,12 +38,19 @@ class KnownDirectivesRule(ASTValidationRule):
         ast_definitions = context.document.definitions
         for def_ in ast_definitions:
             if isinstance(def_, DirectiveDefinitionNode):
-                locations_map[def_.name.value] = [
+                locations_map[def_.name.value] = tuple(
                     DirectiveLocation[name.value] for name in def_.locations
-                ]
+                )
         self.locations_map = locations_map
 
-    def enter_directive(self, node: DirectiveNode, _key, _parent, _path, ancestors):
+    def enter_directive(
+        self,
+        node: DirectiveNode,
+        _key: Any,
+        _parent: Any,
+        _path: Any,
+        ancestors: List[Node],
+    ) -> None:
         name = node.name.value
         locations = self.locations_map.get(name)
         if locations:
@@ -61,14 +58,13 @@ class KnownDirectivesRule(ASTValidationRule):
             if candidate_location and candidate_location not in locations:
                 self.report_error(
                     GraphQLError(
-                        misplaced_directive_message(name, candidate_location.value),
+                        f"Directive '@{name}'"
+                        f" may not be used on {candidate_location.value}.",
                         node,
                     )
                 )
         else:
-            self.report_error(
-                GraphQLError(unknown_directive_message(node.name.value), node)
-            )
+            self.report_error(GraphQLError(f"Unknown directive '@{name}'.", node))
 
 
 _operation_location = {
@@ -102,19 +98,22 @@ _directive_location = {
 }
 
 
-def get_directive_location_for_ast_path(ancestors):
+def get_directive_location_for_ast_path(
+    ancestors: List[Node],
+) -> Optional[DirectiveLocation]:
     applied_to = ancestors[-1]
-    if isinstance(applied_to, Node):
-        kind = applied_to.kind
-        if kind == "operation_definition":
-            applied_to = cast(OperationDefinitionNode, applied_to)
-            return _operation_location.get(applied_to.operation.value)
-        elif kind == "input_value_definition":
-            parent_node = ancestors[-3]
-            return (
-                DirectiveLocation.INPUT_FIELD_DEFINITION
-                if parent_node.kind == "input_object_type_definition"
-                else DirectiveLocation.ARGUMENT_DEFINITION
-            )
-        else:
-            return _directive_location.get(kind)
+    if not isinstance(applied_to, Node):  # pragma: no cover
+        raise TypeError("Unexpected error in directive.")
+    kind = applied_to.kind
+    if kind == "operation_definition":
+        applied_to = cast(OperationDefinitionNode, applied_to)
+        return _operation_location[applied_to.operation.value]
+    elif kind == "input_value_definition":
+        parent_node = ancestors[-3]
+        return (
+            DirectiveLocation.INPUT_FIELD_DEFINITION
+            if parent_node.kind == "input_object_type_definition"
+            else DirectiveLocation.ARGUMENT_DEFINITION
+        )
+    else:
+        return _directive_location.get(kind)
