@@ -1,11 +1,11 @@
-from typing import List, Sequence
+from typing import Collection, List, Optional, Type
 
 from ..error import GraphQLError
-from ..language import DocumentNode, ParallelVisitor, TypeInfoVisitor, visit
+from ..language import DocumentNode, ParallelVisitor, visit
+from ..pyutils import inspect, is_collection
 from ..type import GraphQLSchema, assert_valid_schema
-from ..pyutils import inspect
-from ..utilities import TypeInfo
-from .rules import RuleType
+from ..utilities import TypeInfo, TypeInfoVisitor
+from .rules import ASTValidationRule
 from .specified_rules import specified_rules, specified_sdl_rules
 from .validation_context import SDLValidationContext, ValidationContext
 
@@ -19,9 +19,9 @@ class ValidationAbortedError(RuntimeError):
 def validate(
     schema: GraphQLSchema,
     document_ast: DocumentNode,
-    rules: Sequence[RuleType] = None,
-    type_info: TypeInfo = None,
-    max_errors: int = None,
+    rules: Optional[Collection[Type[ASTValidationRule]]] = None,
+    max_errors: Optional[int] = None,
+    type_info: Optional[TypeInfo] = None,
 ) -> List[GraphQLError]:
     """Implements the "Validation" section of the spec.
 
@@ -35,28 +35,37 @@ def validate(
     a ValidationContext (see the language/visitor API). Visitor methods are expected to
     return GraphQLErrors, or lists of GraphQLErrors when invalid.
 
-    Optionally a custom TypeInfo instance may be provided. If not provided, one will be
-    created from the provided schema.
+    Validate will stop validation after a ``max_errors`` limit has been reached.
+    Attackers can send pathologically invalid queries to induce a DoS attack,
+    so by default ``max_errors`` set to 100 errors.
+
+    Providing a custom TypeInfo instance is deprecated and will be removed in v3.3.
     """
     if not document_ast or not isinstance(document_ast, DocumentNode):
-        raise TypeError("You must provide a document node.")
+        raise TypeError("Must provide document.")
     # If the schema used for validation is invalid, throw an error.
     assert_valid_schema(schema)
+    if max_errors is None:
+        max_errors = 100
+    elif not isinstance(max_errors, int):
+        raise TypeError("The maximum number of errors must be passed as an int.")
     if type_info is None:
         type_info = TypeInfo(schema)
     elif not isinstance(type_info, TypeInfo):
-        raise TypeError(f"Not a TypeInfo object: {inspect(type_info)}")
+        raise TypeError(f"Not a TypeInfo object: {inspect(type_info)}.")
     if rules is None:
         rules = specified_rules
-    elif not isinstance(rules, (list, tuple)):
-        raise TypeError("Rules must be passed as a list/tuple.")
-    if max_errors is not None and not isinstance(max_errors, int):
-        raise TypeError("The maximum number of errors must be passed as an int.")
+    elif not is_collection(rules) or not all(
+        isinstance(rule, type) and issubclass(rule, ASTValidationRule) for rule in rules
+    ):
+        raise TypeError(
+            "Rules must be specified as a collection of ASTValidationRule subclasses."
+        )
 
     errors: List[GraphQLError] = []
 
     def on_error(error: GraphQLError) -> None:
-        if max_errors is not None and len(errors) >= max_errors:
+        if len(errors) >= max_errors:
             errors.append(
                 GraphQLError(
                     "Too many validation errors, error limit reached."
@@ -82,16 +91,20 @@ def validate(
 
 def validate_sdl(
     document_ast: DocumentNode,
-    schema_to_extend: GraphQLSchema = None,
-    rules: Sequence[RuleType] = None,
+    schema_to_extend: Optional[GraphQLSchema] = None,
+    rules: Optional[Collection[Type[ASTValidationRule]]] = None,
 ) -> List[GraphQLError]:
-    """Validate an SDL document."""
-    context = SDLValidationContext(document_ast, schema_to_extend)
+    """Validate an SDL document.
+
+    For internal use only.
+    """
+    errors: List[GraphQLError] = []
+    context = SDLValidationContext(document_ast, schema_to_extend, errors.append)
     if rules is None:
         rules = specified_sdl_rules
     visitors = [rule(context) for rule in rules]
     visit(document_ast, ParallelVisitor(visitors))
-    return context.errors
+    return errors
 
 
 def assert_valid_sdl(document_ast: DocumentNode) -> None:
