@@ -14,27 +14,20 @@ class SampleTestNode(Node):
 
 def describe_token_class():
     def initializes():
-        prev = Token(TokenKind.EQUALS, 10, 11, 1, 2)
         token = Token(
             kind=TokenKind.NAME,
             start=11,
             end=12,
             line=1,
             column=3,
-            prev=prev,
             value="n",
         )
-        assert prev.kind == TokenKind.EQUALS
-        assert prev.start == 10
-        assert prev.end == 11
-        assert prev.line == 1
-        assert prev.column == 2
         assert token.kind == TokenKind.NAME
         assert token.start == 11
         assert token.end == 12
         assert token.line == 1
         assert token.column == 3
-        assert token.prev is prev
+        assert token.prev is None
         assert token.value == "n"
 
     def can_stringify():
@@ -187,6 +180,19 @@ def describe_node_class():
         assert node3 != node
         assert hash(node3) != hash(node)
 
+    def caches_are_hashed():
+        node = SampleTestNode(alpha=1)
+        assert not hasattr(node, "_hash")
+        hash1 = hash(node)
+        assert hasattr(node, "_hash")
+        assert hash1 == getattr(node, "_hash")
+        node.alpha = 2
+        assert not hasattr(node, "_hash")
+        hash2 = hash(node)
+        assert hash2 != hash1
+        assert hasattr(node, "_hash")
+        assert hash2 == getattr(node, "_hash")
+
     def can_create_weak_reference():
         node = SampleTestNode(alpha=1, beta=2)
         ref = weakref.ref(node)
@@ -194,7 +200,7 @@ def describe_node_class():
 
     def can_create_custom_attribute():
         node = SampleTestNode(alpha=1, beta=2)
-        node.gamma = 3  # type: ignore
+        node.gamma = 3
         assert node.gamma == 3  # type: ignore
 
     def can_create_shallow_copy():
@@ -234,4 +240,32 @@ def describe_node_class():
         assert Foo.kind == "foo"
 
     def provides_keys_as_class_attribute():
-        assert SampleTestNode.keys == ["loc", "alpha", "beta"]
+        assert SampleTestNode.keys == ("loc", "alpha", "beta")
+
+    def can_can_convert_to_dict():
+        node = SampleTestNode(alpha=1, beta=2)
+        res = node.to_dict()
+        assert node.to_dict(locations=True) == res
+        assert res == {"kind": "sample_test", "alpha": 1, "beta": 2}
+        assert list(res) == ["kind", "alpha", "beta"]
+
+    def can_can_convert_to_dict_with_locations():
+        token = Token(
+            kind=TokenKind.NAME,
+            start=1,
+            end=3,
+            line=1,
+            column=1,
+            value="foo",
+        )
+        loc = Location(token, token, Source("foo"))
+        node = SampleTestNode(alpha=1, beta=2, loc=loc)
+        res = node.to_dict(locations=True)
+        assert res == {
+            "kind": "sample_test",
+            "alpha": 1,
+            "beta": 2,
+            "loc": {"start": 1, "end": 3},
+        }
+        assert list(res) == ["kind", "alpha", "beta", "loc"]
+        assert list(res["loc"]) == ["start", "end"]
