@@ -58,6 +58,7 @@ def describe_printer_sdl_document():
               five(argument: [String] = ["string", "string"]): String
               six(argument: InputType = {key: "value"}): Type
               seven(argument: Int = null): Type
+              eight(argument: OneOfInputType): Type
             }
 
             type AnnotatedObject @onObject(arg: "value") {
@@ -140,6 +141,11 @@ def describe_printer_sdl_document():
               answer: Int = 42
             }
 
+            input OneOfInputType @oneOf {
+              string: String
+              int: Int
+            }
+
             input AnnotatedInput @onInputObject {
               annotatedField: Type @onInputFieldDefinition
             }
