@@ -1,32 +1,67 @@
 from functools import partial
 
+from graphql.utilities import build_schema
 from graphql.validation import PossibleFragmentSpreadsRule
-from graphql.validation.rules.possible_fragment_spreads import (
-    type_incompatible_spread_message,
-    type_incompatible_anon_spread_message,
-)
 
 from .harness import assert_validation_errors
 
-assert_errors = partial(assert_validation_errors, PossibleFragmentSpreadsRule)
+test_schema = build_schema(
+    """
+    interface Being {
+      name: String
+    }
+
+    interface Pet implements Being {
+      name: String
+    }
+
+    type Dog implements Being & Pet {
+      name: String
+      barkVolume: Int
+    }
+
+    type Cat implements Being & Pet {
+      name: String
+      meowVolume: Int
+    }
+
+    union CatOrDog = Cat | Dog
+
+    interface Intelligent {
+      iq: Int
+    }
+
+    type Human implements Being & Intelligent {
+      name: String
+      pets: [Pet]
+      iq: Int
+    }
+
+    type Alien implements Being & Intelligent {
+      name: String
+      iq: Int
+    }
+
+    union DogOrHuman = Dog | Human
+
+    union HumanOrAlien = Human | Alien
+
+    type Query {
+      catOrDog: CatOrDog
+      dogOrHuman: DogOrHuman
+      humanOrAlien: HumanOrAlien
+    }
+    """
+)
+
+
+assert_errors = partial(
+    assert_validation_errors, PossibleFragmentSpreadsRule, schema=test_schema
+)
 
 assert_valid = partial(assert_errors, errors=[])
 
 
-def error(frag_name, parent_type, frag_type, line, column):
-    return {
-        "message": type_incompatible_spread_message(frag_name, parent_type, frag_type),
-        "locations": [(line, column)],
-    }
-
-
-def error_anon(parent_type, frag_type, line, column):
-    return {
-        "message": type_incompatible_anon_spread_message(parent_type, frag_type),
-        "locations": [(line, column)],
-    }
-
-
 def describe_validate_possible_fragment_spreads():
     def of_the_same_object():
         assert_valid(
@@ -122,13 +157,26 @@ def describe_validate_possible_fragment_spreads():
             """
         )
 
+    def ignores_unknown_fragments_caught_by_known_fragment_names():
+        assert_valid(
+            """
+            fragment petFragment on Pet { ...UnknownFragment }
+            """
+        )
+
     def different_object_into_object():
         assert_errors(
             """
             fragment invalidObjectWithinObject on Cat { ...dogFragment }
             fragment dogFragment on Dog { barkVolume }
             """,
-            [error("dogFragment", "Cat", "Dog", 2, 57)],
+            [
+                {
+                    "message": "Fragment 'dogFragment' cannot be spread here"
+                    " as objects of type 'Cat' can never be of type 'Dog'.",
+                    "locations": [(2, 57)],
+                },
+            ],
         )
 
     def different_object_into_object_in_inline_fragment():
@@ -138,7 +186,13 @@ def describe_validate_possible_fragment_spreads():
               ... on Dog { barkVolume }
             }
             """,
-            [error_anon("Cat", "Dog", 3, 15)],
+            [
+                {
+                    "message": "Fragment cannot be spread here"
+                    " as objects of type 'Cat' can never be of type 'Dog'.",
+                    "locations": [(3, 15)],
+                },
+            ],
         )
 
     def object_into_not_implementing_interface():
@@ -147,7 +201,13 @@ def describe_validate_possible_fragment_spreads():
             fragment invalidObjectWithinInterface on Pet { ...humanFragment }
             fragment humanFragment on Human { pets { name } }
             """,
-            [error("humanFragment", "Pet", "Human", 2, 60)],
+            [
+                {
+                    "message": "Fragment 'humanFragment' cannot be spread here"
+                    " as objects of type 'Pet' can never be of type 'Human'.",
+                    "locations": [(2, 60)],
+                },
+            ],
         )
 
     def object_into_not_containing_union():
@@ -156,7 +216,13 @@ def describe_validate_possible_fragment_spreads():
             fragment invalidObjectWithinUnion on CatOrDog { ...humanFragment }
             fragment humanFragment on Human { pets { name } }
             """,
-            [error("humanFragment", "CatOrDog", "Human", 2, 61)],
+            [
+                {
+                    "message": "Fragment 'humanFragment' cannot be spread here"
+                    " as objects of type 'CatOrDog' can never be of type 'Human'.",
+                    "locations": [(2, 61)],
+                },
+            ],
         )
 
     def union_into_not_contained_object():
@@ -165,7 +231,13 @@ def describe_validate_possible_fragment_spreads():
             fragment invalidUnionWithinObject on Human { ...catOrDogFragment }
             fragment catOrDogFragment on CatOrDog { __typename }
             """,
-            [error("catOrDogFragment", "Human", "CatOrDog", 2, 58)],
+            [
+                {
+                    "message": "Fragment 'catOrDogFragment' cannot be spread here"
+                    " as objects of type 'Human' can never be of type 'CatOrDog'.",
+                    "locations": [(2, 58)],
+                },
+            ],
         )
 
     def union_into_non_overlapping_interface():
@@ -174,7 +246,13 @@ def describe_validate_possible_fragment_spreads():
             fragment invalidUnionWithinInterface on Pet { ...humanOrAlienFragment }
             fragment humanOrAlienFragment on HumanOrAlien { __typename }
             """,
-            [error("humanOrAlienFragment", "Pet", "HumanOrAlien", 2, 59)],
+            [
+                {
+                    "message": "Fragment 'humanOrAlienFragment' cannot be spread here"
+                    " as objects of type 'Pet' can never be of type 'HumanOrAlien'.",
+                    "locations": [(2, 59)],
+                },
+            ],
         )
 
     def union_into_non_overlapping_union():
@@ -183,7 +261,14 @@ def describe_validate_possible_fragment_spreads():
             fragment invalidUnionWithinUnion on CatOrDog { ...humanOrAlienFragment }
             fragment humanOrAlienFragment on HumanOrAlien { __typename }
             """,
-            [error("humanOrAlienFragment", "CatOrDog", "HumanOrAlien", 2, 60)],
+            [
+                {
+                    "message": "Fragment 'humanOrAlienFragment'"
+                    " cannot be spread here as objects of type 'CatOrDog'"
+                    " can never be of type 'HumanOrAlien'.",
+                    "locations": [(2, 60)],
+                },
+            ],
         )
 
     def interface_into_non_implementing_object():
@@ -192,7 +277,13 @@ def describe_validate_possible_fragment_spreads():
             fragment invalidInterfaceWithinObject on Cat { ...intelligentFragment }
             fragment intelligentFragment on Intelligent { iq }
             """,
-            [error("intelligentFragment", "Cat", "Intelligent", 2, 60)],
+            [
+                {
+                    "message": "Fragment 'intelligentFragment' cannot be spread here"
+                    " as objects of type 'Cat' can never be of type 'Intelligent'.",
+                    "locations": [(2, 60)],
+                },
+            ],
         )
 
     def interface_into_non_overlapping_interface():
@@ -203,7 +294,13 @@ def describe_validate_possible_fragment_spreads():
             }
             fragment intelligentFragment on Intelligent { iq }
             """,
-            [error("intelligentFragment", "Pet", "Intelligent", 3, 15)],
+            [
+                {
+                    "message": "Fragment 'intelligentFragment' cannot be spread here"
+                    " as objects of type 'Pet' can never be of type 'Intelligent'.",
+                    "locations": [(3, 15)],
+                },
+            ],
         )
 
     def interface_into_non_overlapping_interface_in_inline_fragment():
@@ -213,7 +310,13 @@ def describe_validate_possible_fragment_spreads():
               ...on Intelligent { iq }
             }
             """,
-            [error_anon("Pet", "Intelligent", 3, 15)],
+            [
+                {
+                    "message": "Fragment cannot be spread here as objects"
+                    " of type 'Pet' can never be of type 'Intelligent'.",
+                    "locations": [(3, 15)],
+                },
+            ],
         )
 
     def interface_into_non_overlapping_union():
@@ -222,5 +325,11 @@ def describe_validate_possible_fragment_spreads():
             fragment invalidInterfaceWithinUnion on HumanOrAlien { ...petFragment }
             fragment petFragment on Pet { name }
             """,
-            [error("petFragment", "HumanOrAlien", "Pet", 2, 68)],
+            [
+                {
+                    "message": "Fragment 'petFragment' cannot be spread here"
+                    " as objects of type 'HumanOrAlien' can never be of type 'Pet'.",
+                    "locations": [(2, 68)],
+                },
+            ],
         )
